<?php

namespace Drupal\inline_translation;

use Drupal\content_translation\ContentTranslationManagerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a class for reacting to entity events.
 *
 * @internal
 */
class EntityOperations implements ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The content translation manager service.
   *
   * @var \Drupal\content_translation\ContentTranslationManagerInterface
   */
  protected $contentTranslationManager;

  /**
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * EntityOperations constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\content_translation\ContentTranslationManagerInterface $content_translation_manager
   *   The content translation manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ContentTranslationManagerInterface $content_translation_manager, LanguageManagerInterface $language_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->contentTranslationManager = $content_translation_manager;
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('content_translation.manager'),
      $container->get('language_manager')
    );
  }

  /**
   * Alters entity forms to disallow concurrent editing in multiple workspaces.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param string $form_id
   *   The form ID.
   *
   * @see hook_form_alter()
   */
  public function entityFormAlter(array &$form, FormStateInterface $form_state, $form_id) {
    /** @var \Drupal\Core\Entity\RevisionableInterface $entity */
    $entity = $form_state->getFormObject()->getEntity();
    if (!$this->isSupported($entity)) {
      return;
    }

    // Inject our submit handler immediately before the entity save function
    // If we don't find the save function, don't do anything
    $key = array_search('::save', $form['actions']['submit']['#submit']);
    if ($key !== FALSE) {
      // array_splice($form['actions']['submit']['#submit'], $key, 0, 'inline_translation_entity_form_submit');
      // @todo Fix the save order without unset.
      unset($form['actions']['submit']['#submit'][$key]);
      $form['actions']['submit']['#submit'][] = [$this, 'entityFormSubmit'];
      $form['actions']['submit']['#submit'][] = '::save';
      // Add entity translation form elements for unified language node form.
      $this->entityFormAddField($form, $form_state);
    }
  }

  /**
   * Checks if inline translation is enabled.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   *
   * @return bool
   *   Returns true if inline translation enabled, FALSE otherwise.
   */
  protected function isSupported($entity) {
    $entity_type_id = $entity->getEntityType()->id();
    $bundle = $entity->bundle();
    $original = $entity->getUntranslated()->language()->getId();

    // @todo return the inline translation settings of the bundle.
    $settings = $this->contentTranslationManager->getBundleTranslationSettings($entity_type_id, $bundle);

    // 1. check translation is enabled.
    $is_enabled = $this->contentTranslationManager->isEnabled($entity_type_id, $bundle);

    // 2. Check if the current language is the original.
    $is_default_language = $this->languageManager->getCurrentLanguage()->getId() == $original;

    return $is_enabled && $is_default_language;
  }

  /**
   * Entity form alter.
   *
   * @param array $form
   *   An array of form elements.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form stateobject.
   */
  protected function entityFormAddField(&$form, FormStateInterface $form_state) {
    $entity = $form_state->getFormObject()->getEntity();
    $entity_type = $entity->getEntityTypeId();
    $bundle = $entity->bundle();

    $entityManager = \Drupal::service('entity_field.manager');
    $fields = $entityManager->getFieldDefinitions($entity_type, $bundle);
    foreach ($fields as $field_name => $field_definition) {
      $this->addLanguageElements($form, $form_state, $field_definition);
    }
  }

  /**
   * Add all enabled language fields for a single field.
   */
  protected function addLanguageElements(&$form, &$form_state, $field_definition) {
    $language_manager = $this->languageManager;
    $entity = $form_state->getFormObject()->getEntity();
    $form_display = EntityFormDisplay::collectRenderDisplay($entity, 'default');
    $display_options = $field_definition->getDisplayOptions('form');
    $hidden = empty($display_options) || (isset($display_options['type']) && $display_options['type'] == 'hidden');

    // Only process entity-translatable fields.
    if ($field_definition->isTranslatable() && !$hidden) {
      $field_name = $field_definition->getName();

      $widget = $form_display->getRenderer($field_name);
      $items = $entity->get($field_name);
      $field_form = $widget->form($items, $form, $form_state);
      $widget_form = $field_form['widget'];
      unset($field_form['widget']);
      $field_form['widget']['group'] = [
        '#type' => 'horizontal_tabs',
      ];

      foreach ($language_manager->getLanguages() as $language) {
        $langcode = $language->getId();

        $field_form['widget']['group'][$langcode] = [
          '#type' => 'details',
          '#title' => $language->getName(),
          '#group' => 'inline',
          '#tree' => TRUE,
        ];
        $field_form['widget']['group'][$langcode]['elements'] = $widget_form;
        if ($langcode != $language_manager->getCurrentLanguage()->getId()) {
          // Override language specific values, when available.
          if ($entity->hasTranslation($langcode)) {
            $language_items = $entity->getTranslation($langcode)->get($field_name);
            $language_field_form = $widget->form($language_items, $form, $form_state);
            $language_widget_form = $language_field_form['widget'];
            $field_form['widget']['group'][$langcode]['elements'] = $language_widget_form;
          }
          $field_form['widget']['group'][$langcode]['elements']['#parents'][] = $langcode;
        }
      }
      $form[$field_name] = $field_form;
      $form[$field_name]['#multilingual'] = TRUE;
    }
  }

  /**
   * Form submission handler.
   *
   */
  public function entityFormSubmit($form, FormStateInterface $form_state) {
    /** @var \Drupal\Core\Entity\ContentEntityFormInterface $form_object */
    $form_object = $form_state->getFormObject();
    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $entity = $form_object->getEntity();
    $entity_type = $entity->getEntityTypeId();
    $bundle = $entity->bundle();
    $current_language = \Drupal::languageManager()->getCurrentLanguage();
    $languages = \Drupal::languageManager()->getLanguages();

    $entityManager = \Drupal::service('entity_field.manager');
    $fields = $entityManager->getFieldDefinitions($entity_type, $bundle);

    // Process the translated values
    $values = $form_state->getValues();

    foreach ($languages as $langcode => $language) {
      if ($langcode == $current_language->getId()) {
        continue;
      }
      // get the existing translation, or create one if it doesn't currently exist
      if($entity->hasTranslation($langcode)) {
        $translation = $entity->getTranslation($langcode);
      }
      else {
        $translation = $entity->addTranslation($langcode);
      }

      // Copy the translated values from form_state to the entity translation
      foreach ($fields as $field_name => $field_definition) {
        if ($field_name == 'langcode') {
          continue;
        }
        if(isset($values[$field_name][$langcode])) {
          $translation->set($field_name, $values[$field_name][$langcode]);
          unset($values[$field_name][$langcode]);
        }
      }
    }
  }

}
